<?php

namespace App\Controller\web;

use App\Controller\UploadTrait;

class LegalProofAjaxController
{
    use UploadTrait;

    /**
     * $config.
     *
     * @var array
     */
    protected $config;

    public function __construct()
    {
        if (!\Request::ajax()) {
            throw new \Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException(['ajax']);
        }

        $this->config = config('contact');
    }

    /**
     * 多檔上傳
     */
    public function ajax_legal_proof_upload()
    {
        // dd(\Request::file('legal_proof'));
        $rule = [
            'legal_proof.*' => 'mimes:'.$this->config['legal_proof_mime'].'|max:'.$this->config['legal_proof_max_limit'],
            'mid' => [
                function($attribute, $value, $fail) {
                    $mid = $value;
                    $count = \DB::table($this->config['legal_proof_table'])
                               ->where([ ['session_id', \Session::getId()]])
                                ->count();

                    if ($count >= 5) {
                        return $fail('最多上傳 5 個文件');
                    }
                }
            ],
        ];
        $rule_message = [
            'legal_proof.*.mimes' => ' 檔案格式有誤 限制：:values',
            'legal_proof.*.max'   => ' 檔案大小不可超過 '.get_file_max($this->config['legal_proof_max_limit']),
        ];
        // 驗證器
        validate($rule, $rule_message);

        $message = ['success' => '', 'error' => ''];

        if(\Request::hasFile('legal_proof')) {
            foreach (\Request::file('legal_proof') as $key => $file) {
                $file_path = $this->file_upload($file, $this->config['folder']);

                if(
                    \DB::table($this->config['legal_proof_table'])->insert([
                        // 'mid'        => \Request::input('mid'),
                        'file'       => $file_path,
                        'file_type'  => $file->getClientOriginalExtension(),
                        'name'       => str_replace('.' . $file->getClientOriginalExtension(), '', $file->getClientOriginalName()),
                        'session_id' => \Session::getId(),
                    ])
                ) {
                    $message['success'] .= $file->getClientOriginalName() .' 新增成功 ';
                } else {
                    $message['error'] .= $file->getClientOriginalName() .' 新增失敗 ';
                }
            }
        }

        return \JsonResponse::create($message)->send();
    }

    /**
     * 取得多檔資料
     */
    public function ajax_legal_proof_get($id)
    {
        
            $where = [['session_id', \Session::getId()]];
        if($id == 0) { // 新增時 顯示暫存
            $where = [['session_id', \Session::getId()]];
        } else {
            $where = [['contact_id', $id]];
        }

        $data = \DB::table($this->config['legal_proof_table'])
                   ->where($where)
                   ->orderBy('no', 'desc')
                   ->orderBy('id', 'asc')
                   ->get();

        $download_table = $this->config['legal_proof_table'];

        $html = \View::make("web.{$this->config['folder']}.legal_proof", compact('data', 'download_table'))->render();

        return \JsonResponse::create(['html' => $html])->send();
    }

    /**
     * 多檔更新 選擇器
     */
    public function ajax_legal_proof_upd( $id, $column)
    {
        switch ($column) {
            case 'no':
                $this->ajax_legal_proof_upd_no($id, $column);
                break;
            case 'name':
                $this->ajax_legal_proof_upd_name($id, $column);
                break;
            default:
                if (
                    \DB::table($this->config['legal_proof_table'])->where('id', $id)->update([$column => \Request::input('value')])
                ) {
                    $message['success'] = '更新成功';
                } else {
                    $message['error'] = '更新失敗,欄位參數：' . $column;
                }

                return \JsonResponse::create($message)->send();
                break;
        }
    }

    /**
     * 多檔更新排序
     */
    protected function ajax_legal_proof_upd_no($id, $column)
    {
        $rule = ['value' => 'filled|numeric'];
        $rule_message = [
            'value.filled'  => '排序 不能為空值',
            'value.numeric' => '排序 請輸入數字',
        ];
        // 驗證器
        validate($rule, $rule_message);

        $message = ['success' => '', 'error' => ''];

        if(
            \DB::table($this->config['legal_proof_table'])->where('id', $id)->update([$column => \Request::input('value')])
        ) {
            $message['success'] = '更新成功';
        } else {
            $message['error'] = '更新失敗';
        }

        return \JsonResponse::create($message)->send();
    }

    /**
     * 多檔更新名稱
     */
    protected function ajax_legal_proof_upd_name($id, $column)
    {
        $message =['success' => '', 'error' => ''];

        if(
            \DB::table($this->config['legal_proof_table'])->where('id', $id)->update([$column => \Request::input('value')])
        ) {
            $message['success'] = '更新成功';
        } else {
            $message['error'] = '更新失敗';
        }

        return \JsonResponse::create($message)->send();
    }

    /**
     * 多檔刪除
     */
    public function ajax_legal_proof_del( $id)
    {
        $message =['success' => '', 'error' => ''];

        $path = \DB::table($this->config['legal_proof_table'])->where('id', $id)->value('file');
        \Storage::delete(str_replace(storage_path(), '', $path));

        if(\DB::table($this->config['legal_proof_table'])->where('id', $id)->delete()) {
            $message['success'] = '刪除成功';
        } else {
            $message['error'] = '刪除失敗';
        }

        return \JsonResponse::create($message)->send();
    }
}